@@ -1,7 +1,7 @@
 package Complete::Util;
 
-our $DATE = '2015-12-17'; # DATE
-our $VERSION = '0.42'; # VERSION
+our $DATE = '2016-01-05'; # DATE
+our $VERSION = '0.43'; # VERSION
 
 use 5.010001;
 use strict;
@@ -145,8 +145,29 @@ Will sort the resulting completion list, so you don't have to presort the array.
 _
     args => {
         %arg_word,
-        array     => { schema=>['array*'=>{of=>'str*'}], req=>1 },
-        exclude   => { schema=>['array*'] },
+        array       => {
+            schema => ['array*'=>{of=>'str*'}],
+            req => 1,
+        },
+        exclude     => {
+            schema => ['array*'],
+        },
+        replace_map => {
+            schema => ['hash*', each_value=>['array*', of=>'str*']],
+            description => <<'_',
+
+You can supply correction entries in this option. An example is when array if
+`['mount','unmount']` and `umount` is a popular "typo" for `unmount`. When
+someone already types `um` it cannot be completed into anything (even the
+current fuzzy mode will return *both* so it cannot complete immediately).
+
+One solution is to add replace_map `{'unmount'=>['umount']}`. This way, `umount`
+will be regarded the same as `unmount` and when user types `um` it can be
+completed unambiguously into `unmount`.
+
+_
+            tags => ['experimental'],
+        },
     },
     result_naked => 1,
     result => {
@@ -171,13 +192,51 @@ sub complete_array_elem {
     # normalize
     my $wordn = $ci ? uc($word) : $word; $wordn =~ s/_/-/g if $map_case;
 
-    my @words;
-    my @arrayn;
+    my $excluden;
+    if ($args{exclude}) {
+        $excluden = {};
+        for my $el (@{$args{exclude}}) {
+            my $eln = $ci ? uc($el) : $el; $eln =~ s/_/-/g if $map_case;
+            $excluden->{$eln} //= 1;
+        }
+    }
+
+    my $rmapn;
+    my $rev_rmapn; # to replace back to the original words back in the result
+    if (my $rmap = $args{replace_map}) {
+        $rmapn = {};
+        $rev_rmapn = {};
+        for my $k (keys %$rmap) {
+            my $kn = $ci ? uc($k) : $k; $kn =~ s/_/-/g if $map_case;
+            my @vn;
+            for my $v (@{ $rmap->{$k} }) {
+                my $vn = $ci ? uc($v) : $v; $vn =~ s/_/-/g if $map_case;
+                push @vn, $vn;
+                $rev_rmapn->{$vn} //= $k;
+            }
+            $rmapn->{$kn} = \@vn;
+        }
+    }
+
+    my @words; # the answer
+    my @arrayn; # case- & map-case-normalized form of $array (+rmap entries)
+
+    # normal (non-fuzzy, non-word-mode) matching. we also fill @arrayn here
+    # (which will be used again in word-mode & fuzzy matching) so we don't have
+    # to calculate again.
     for my $el (@$array) {
         my $eln = $ci ? uc($el) : $el; $eln =~ s/_/-/g if $map_case;
-        push @arrayn, $eln; # so we don't have to calculate again
-        next unless 0==index($eln, $wordn);
-        push @words, $el;
+        next if $excluden && $excluden->{$eln};
+        push @arrayn, $eln;
+        push @words, $el if 0==index($eln, $wordn);
+        if ($rmapn && $rmapn->{$eln}) {
+            for my $vn (@{ $rmapn->{$eln} }) {
+                push @arrayn, $vn;
+                # we add the normalized form, because we'll just revert it back
+                # to the original word in the final result
+                push @words, $vn if 0==index($vn, $wordn);
+            }
+        }
     }
 
     {
@@ -255,12 +314,18 @@ sub complete_array_elem {
         }
     }
 
-    if ($args{exclude}) {
-        my $exclude = $ci ? [map {uc} @{ $args{exclude} }] : $args{exclude};
-        @words = grep {
-            my $w = $_;
-            !(grep {($ci ? uc($w) : $w) eq $_} @$exclude);
-        } @words;
+    # replace back the words from replace_map
+    if ($rmapn && @words) {
+        my @wordsn;
+        for my $el (@words) {
+            my $eln = $ci ? uc($el) : $el; $eln =~ s/_/-/g if $map_case;
+            push @wordsn, $eln;
+        }
+        for my $i (0..$#words) {
+            if (my $w = $rev_rmapn->{$wordsn[$i]}) {
+                $words[$i] = $w;
+            }
+        }
     }
 
     return $ci ? [sort {lc($a) cmp lc($b)} @words] : [sort @words];
@@ -412,7 +477,7 @@ Complete::Util - General completion routine
 
 =head1 VERSION
 
-This document describes version 0.42 of Complete::Util (from Perl distribution Complete-Util), released on 2015-12-17.
+This document describes version 0.43 of Complete::Util (from Perl distribution Complete-Util), released on 2016-01-05.
 
 =head1 DESCRIPTION
 
@@ -490,6 +555,17 @@ Arguments ('*' denotes required arguments):
 
 =item * B<exclude> => I<array>
 
+=item * B<replace_map> => I<hash>
+
+You can supply correction entries in this option. An example is when array if
+C<['mount','unmount']> and C<umount> is a popular "typo" for C<unmount>. When
+someone already types C<um> it cannot be completed into anything (even the
+current fuzzy mode will return I<both> so it cannot complete immediately).
+
+One solution is to add replace_map C<< {'unmount'=E<gt>['umount']} >>. This way, C<umount>
+will be regarded the same as C<unmount> and when user types C<um> it can be
+completed unambiguously into C<unmount>.
+
 =item * B<word>* => I<str> (default: "")
 
 Word to complete.
@@ -561,7 +637,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Complete-U
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Complete-Util>.
+Source repository is at L<https://github.com/sharyanto/perl-SHARYANTO-Complete-Util>.
 
 =head1 BUGS
 
@@ -577,7 +653,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
